package thesis.jndi.provider.api;

import javax.naming.spi.NamingManager;
import javax.naming.spi.StateFactory;
import javax.naming.Name;
import javax.naming.Context;
import javax.naming.NamingException;

import java.util.Hashtable;

/* Because there is no way to register something like state factory builder in NamingManager we'll need this class to
 * connect to ProviderResolver and find the appropriate StateFactory implementation
 * @author Elitsa Pancheva
 */
public class DispatchStateFactory implements StateFactory {
  /**
   * This constant holds the colon-separated list of state factory names
   * to be specified in the environment and to be resolved from the ProviderManager
   */
  public static final String EXTRA_STATE_FACTORIES = "thesis.jndi.provider.state.factories";

  public Object getStateToBind(Object obj, Name name, Context nameCtx, Hashtable environment) throws NamingException {
    //System.out.println("[DispatchStateFactory] getStateToBind name = " + name);
  	// check first if there is registered ProviderManager
  	if (!NamingManager.hasInitialContextFactoryBuilder()) {
  		return null;
  	}
    try {
      return ProviderManager.getInstance().getStateToBind(obj, name, nameCtx, environment);
    } catch (NamingException e) {
      //e.printStackTrace();
      // could be that ProviderResolver is not initialized => return null to NamingManager
      return null;
    }
  }
}
