package thesis.jndi.provider.api;

import javax.naming.spi.StateFactory;
import javax.naming.NamingException;

/* Registers providers of some or all of the following: InitialContextFactory, ObjectFactory, URLContextFactory, StateFactory 
 * implementations and is used in resolving implementations of these factories by given factory name. 
 * Implements StateFactory in order to be used as something like a state factory builder.
 * 
 * @author Elitsa Pancheva
 */
public abstract class ProviderManager implements StateFactory {

  protected static ProviderManager instance = null;

  /**
   * Get an instance of the ProviderManager
   */
  public static ProviderManager getInstance() throws NamingException {
    if (instance == null) {
        throw new NamingException("ProviderManager is not initialized.");
    }
    return instance;
  }

  /**
   * Register a naming or semantics provider
   */
  public abstract void registerProvider(Provider provider) throws NamingException;
  
  /**
   * Unregister specified naming or semantics provider
   */
  public abstract void unregisterProvider(Provider provider) throws NamingException;

}



