package thesis.jndi.provider.impl.semantics;

import javax.naming.spi.ObjectFactoryBuilder;
import javax.naming.spi.ObjectFactory;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.Reference;
import javax.naming.Referenceable;
import javax.naming.RefAddr;
import javax.naming.StringRefAddr;

import thesis.jndi.provider.impl.ProviderResolver;
import thesis.jndi.provider.impl.context.DefaultInitialContext;

import java.util.Hashtable;
import java.util.StringTokenizer;

/* ObjectFactoryBuilder Implementation
 *
 * @author Elitsa Pancheva
 */
public class ObjectFactoryBuilderImpl implements ObjectFactoryBuilder {

  private ObjectFactory defaultOF = new DefaultObjectFactory();

  public ObjectFactory createObjectFactory(Object refInfo, Hashtable env) throws NamingException {
//    System.out.println("ObjectFactoryBuilderImpl.createObjectFactory env = " + env);
    // Use reference if possible
	Reference ref = null;
	if (refInfo instanceof Reference) {
      ref = (Reference) refInfo;
    } else if (refInfo instanceof Referenceable) {
      ref = ((Referenceable)(refInfo)).getReference();
    }

    ProviderResolver mgr = ((ProviderResolver) ProviderResolver.getInstance());
    ObjectFactory fac = null;

    if (ref != null) {
      // if it is a reference
	    // we take the information from it
      String f = ref.getFactoryClassName();
      if (f != null) {
        // if reference identifies a factory, use exclusively
        try {
	        fac =  mgr.findObjectFactory(f);
        } catch (Exception e) {
          NamingException ne = new NamingException("Exception while trying to load factory with name : " + f);
          ne.setRootCause(e);
          throw ne;
        }

        if (fac != null) {
          return fac;
        }
        // No factory found, so return a dummy factory that
        // simply returns refInfo
        return new DefaultObjectFactory(refInfo);

      } else {
        // if reference has no factory, check for addresses
        // containing URLs and try URL context factories.
        RefAddr addr;
        String url, scheme;
        for (int i = 0; i < ref.size(); i++) {
          addr = ref.get(i);
          if ((addr instanceof StringRefAddr) && (addr.getType().equalsIgnoreCase("URL"))) {
            url = (String)addr.getContent();
            if (url != null) {
              scheme = DefaultInitialContext._getURLScheme(url);
              try {
                fac = mgr.findURLContextFactory(scheme, env);
                if (fac!=null) {
                  return fac;
                }
              } catch (NamingException ne) {
                System.out.println("error while iterating for URL context factory search (IGNORED). Exception is : " + ne);
              }
            }
          }
        }
      }
    }
    // ok, even if it was a reference, it was not helping us.
    // we try to go by the environment.
    if (env == null) {
      return defaultOF;
    }
    String objFacs = (String) env.get(Context.OBJECT_FACTORIES);
    if (objFacs != null) {
      StringTokenizer tokenizer = new StringTokenizer(objFacs, ":");
      String objFac;
      ObjectFactory of;
      while (tokenizer.hasMoreTokens()) {
        objFac = tokenizer.nextToken().trim();
        try {
          of = mgr.findObjectFactory(objFac);
          if (of != null) {
            return of;
          }
        } catch (Exception ne) {
          System.out.println("error while iterating over object factories: " +ne+ " . Exception is : " + ne);
          NamingException e = new NamingException("No Object Factory found for "+Context.OBJECT_FACTORIES+"="+objFacs);
          e.setRootCause(ne);
          throw e;
        }
      }
      // we tried all we could... give up!
      System.out.println("No Object Factory found for " + Context.OBJECT_FACTORIES+"="+objFacs  +  " => will return the default");
      //throw new NamingException("No Object Factory found for " + Context.OBJECT_FACTORIES+"="+objFacs);
    }
	  return defaultOF;
  }
}
