package test.thesis.jndi.correctness;

import java.util.Hashtable;
import java.util.Properties;

import javax.naming.CompositeName;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.spi.InitialContextFactory;
import javax.naming.spi.InitialContextFactoryBuilder;
import javax.naming.spi.ObjectFactory;
import javax.naming.spi.ObjectFactoryBuilder;

import test.thesis.jndi.provider.client.TestClient;
import test.thesis.jndi.provider.context.InMemoryContext;
import test.thesis.jndi.provider.context.ServiceSupportContext;
import test.thesis.jndi.provider.semantics.book.Book;
import test.thesis.jndi.provider.semantics.book.BookInfo;
import test.thesis.jndi.provider.semantics.book.BookObjectFactory;
import thesis.jndi.provider.api.DispatchStateFactory;
import thesis.jndi.provider.impl.ProviderResolver;
import thesis.jndi.provider.impl.context.DefaultInitialContext;
import thesis.jndi.provider.impl.context.DefaultInitialContextFactory;
import thesis.jndi.provider.impl.context.InitialContextFactoryBuilderImpl;
import thesis.jndi.provider.impl.semantics.DefaultObjectFactory;
import thesis.jndi.provider.impl.semantics.ObjectFactoryBuilderImpl;
import junit.framework.AssertionFailedError;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

public class FunctionalityTest extends TestCase {
		
	static {
		try {
			ProviderResolver.init();
			TestClient.registerProviders();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
//		env.put(Context.URL_PKG_PREFIXES, "test.thesis.jndi.provider.context");
	}
	
	public FunctionalityTest(String test) {
		super(test);
	}
	
	public void testFindExistingInitialCtx() {
		System.out.println("Find testFindExistingInitialCtx");
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, "test.thesis.jndi.provider.context.InitialInMemoryContextFactory");
		try {
		  InitialContextFactoryBuilder builder = new InitialContextFactoryBuilderImpl();
		  InitialContextFactory icf = builder.createInitialContextFactory(env);
		  assertTrue(icf instanceof DefaultInitialContextFactory);
		  Context ctx = icf.getInitialContext(env);
		  assertTrue(ctx instanceof DefaultInitialContext);
		  ctx = (Context) ctx.lookup("");
		  assertTrue(ctx instanceof InMemoryContext);
		  
		  ctx.createSubcontext("not_valid");
		} catch (Exception e) {
			e.printStackTrace();
			throw new AssertionFailedError("Failed to execute testFindInitialContext because: " + e);
		}
		
	}
	
	public void testFindNotExistingInitialCtx() {
		System.out.println("testFindNotExistingInitialCtx");
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, "test.thesis.jndi.provider.context.NotExistingContextFactory");
		try {
		  InitialContextFactoryBuilder builder = new InitialContextFactoryBuilderImpl();
		  InitialContextFactory icf = builder.createInitialContextFactory(env);
		  assertTrue(icf instanceof DefaultInitialContextFactory);
		  Context ctx = icf.getInitialContext(env);
		  assertTrue(ctx instanceof DefaultInitialContext);
		  ctx = (Context) ctx.lookup("");
		  throw new AssertionFailedError("NamingException was expected here");
	 } catch (Exception e) {
			e.printStackTrace();
			assertTrue(e instanceof NamingException);
	  }
		
	}
		
	public void testFindExistingURLCtx() {
		System.out.println("testFindExistingURLCtx");
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, "test.thesis.jndi.provider.context.InitialInMemoryContextFactory");

		try {
		  InitialContextFactoryBuilder builder = new InitialContextFactoryBuilderImpl();
		  InitialContextFactory icf = builder.createInitialContextFactory(env);
		  assertTrue(icf instanceof DefaultInitialContextFactory);
		  Context ctx = icf.getInitialContext(env);
		  assertTrue(ctx instanceof DefaultInitialContext);
		  ctx = (Context) ctx.lookup("service:/");
		  System.out.println(ctx);
		  assertTrue(ctx instanceof InMemoryContext);
		} catch (Exception e) {
			e.printStackTrace();
			throw new AssertionFailedError("Failed to execute testFindInitialContext because: " + e);
		}
		
	}
	
	public void testFindNotExistingURLCtxAndExistingInitialCtx() {
		System.out.println("testFindNotExistingURLCtxAndExistingInitialCtx");
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, "test.thesis.jndi.provider.context.InitialInMemoryContextFactory");
		
		try {
		  InitialContextFactoryBuilder builder = new InitialContextFactoryBuilderImpl();
		  InitialContextFactory icf = builder.createInitialContextFactory(env);
		  assertTrue(icf instanceof DefaultInitialContextFactory);
		  Context ctx = icf.getInitialContext(env);
		  assertTrue(ctx instanceof DefaultInitialContext);
		  ctx = (Context) ctx.lookup("not_valid:/");
		  assertTrue(ctx instanceof InMemoryContext);
		} catch (Exception e) {
			e.printStackTrace();
			throw new AssertionFailedError("Failed to execute testFindInitialContext because: " + e);
		}		
	}
	
	public void testFindNotExistingURLCtxAndNotExistingInitialCtx() {
		System.out.println("testFindNotExistingURLCtxAndNotExistingInitialCtx");
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, "test.thesis.jndi.provider.context.NotExistingContextFactory");
		try {
		  InitialContextFactoryBuilder builder = new InitialContextFactoryBuilderImpl();
		  InitialContextFactory icf = builder.createInitialContextFactory(env);
		  assertTrue(icf instanceof DefaultInitialContextFactory);
		  Context ctx = icf.getInitialContext(env);
		  assertTrue(ctx instanceof DefaultInitialContext);
		  ctx = (Context) ctx.lookup("not_valid:/");
		  throw new AssertionFailedError("NamingException was expected here");
	 } catch (Exception e) {
			e.printStackTrace();
			assertTrue(e instanceof NamingException);
	  }
		
	}
	
	public void testFindExistingObjectFactory() {
		System.out.println("testFindObjectFactory");
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, "test.thesis.jndi.provider.context.InitialInMemoryContextFactory");
		env.put(Context.OBJECT_FACTORIES, "test.thesis.jndi.provider.semantics.book.BookObjectFactory");
		Properties bookInfo = new Properties();
		bookInfo.put(BookInfo.BOOK_INFO_PROPERTIES, "true");
		
		try {
		  ObjectFactoryBuilder builder = new ObjectFactoryBuilderImpl();
		  ObjectFactory of = builder.createObjectFactory(bookInfo, env);
		  assertTrue(of instanceof BookObjectFactory);
		} catch (Exception e) {
			e.printStackTrace();
			throw new AssertionFailedError("Failed to execute testFindInitialContext because: " + e);
		}
		
	}
	
	public void testFindNotExistingObjectFactory() {
		System.out.println("testFindNotExistingObjectFactory");
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, "test.thesis.jndi.provider.context.InitialInMemoryContextFactory");
		env.put(Context.OBJECT_FACTORIES, "test.thesis.jndi.provider.semantics.book.NotExistingObjectFactory");
		Properties bookInfo = new Properties();
		bookInfo.put(BookInfo.BOOK_INFO_PROPERTIES, "true");
		
		try {
		  ObjectFactoryBuilder builder = new ObjectFactoryBuilderImpl();
		  ObjectFactory of = builder.createObjectFactory(bookInfo, env);
		  System.out.println(of);
		  assertTrue(of instanceof DefaultObjectFactory);
		} catch (Exception e) {
			e.printStackTrace();
			throw new AssertionFailedError("Failed to execute testFindInitialContext because: " + e);
		}
		
	}
	
	public void testFindExistingStateFactory() {
		System.out.println("testFindObjectFactory");
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, "test.thesis.jndi.provider.context.InitialInMemoryContextFactory");
		env.put(DispatchStateFactory.EXTRA_STATE_FACTORIES, "test.thesis.jndi.provider.semantics.book.BookStateFactory");
				
		try {
		  DispatchStateFactory dispatch = new DispatchStateFactory();
		  Object o = dispatch.getStateToBind(new Book(new BookInfo(new Properties())), new CompositeName("book"), null, env);
		  assertTrue(o instanceof Properties);
		} catch (Exception e) {
			e.printStackTrace();
			throw new AssertionFailedError("Failed to execute testFindInitialContext because: " + e);
		}
		
	}
	
	public void testFindNotExistingStateFactory() {
		System.out.println("testFindNotExistingStateFactory");
		Hashtable env = new Hashtable();
		env.put(Context.INITIAL_CONTEXT_FACTORY, "test.thesis.jndi.provider.context.InitialInMemoryContextFactory");
		env.put(DispatchStateFactory.EXTRA_STATE_FACTORIES, "test.thesis.jndi.provider.semantics.book.NotExistingStateFactory");
				
		try {
		  DispatchStateFactory dispatch = new DispatchStateFactory();
		  Object o = dispatch.getStateToBind(new Book(new BookInfo(new Properties())), new CompositeName("book"), null, env);
		  System.out.println(o);
		  assertNull("Result returned should be null", o);
		} catch (Exception e) {
			e.printStackTrace();
			throw new AssertionFailedError("Failed to execute testFindInitialContext because: " + e);
		}		
	}
	
	public static Test suite(){	
		TestSuite suite = new TestSuite(); 
		suite.addTest(new FunctionalityTest("testFindExistingInitialCtx"));
		suite.addTest(new FunctionalityTest("testFindNotExistingInitialCtx"));
		suite.addTest(new FunctionalityTest("testFindExistingURLCtx"));
		suite.addTest(new FunctionalityTest("testFindNotExistingURLCtxAndExistingInitialCtx"));
		suite.addTest(new FunctionalityTest("testFindNotExistingURLCtxAndNotExistingInitialCtx"));
		suite.addTest(new FunctionalityTest("testFindExistingObjectFactory"));
		suite.addTest(new FunctionalityTest("testFindNotExistingObjectFactory"));
		suite.addTest(new FunctionalityTest("testFindExistingStateFactory"));
		suite.addTest(new FunctionalityTest("testFindNotExistingStateFactory"));
		return suite;
	}
	
}
