package test.thesis.jndi.provider.client;

import java.util.Hashtable;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

import junit.framework.TestCase;

import test.thesis.jndi.provider.context.InMemoryContextProvider;
import test.thesis.jndi.provider.semantics.book.Book;
import test.thesis.jndi.provider.semantics.book.BookInfo;
import test.thesis.jndi.provider.semantics.book.BookProvider;
import test.thesis.jndi.provider.semantics.song.Song;
import test.thesis.jndi.provider.semantics.song.SongInfo;
import thesis.jndi.provider.api.DispatchStateFactory;
import thesis.jndi.provider.api.ProviderManager;

public class TestClient extends TestCase {
  
  private String fakeOF1 = "aaa.aaa.aaaaa.AAAAA";
  private String fakeOF2 = "b.bb.bbb.bbbb.BB";
  
  private String fakeURLpkg3 = "ccccccc.cc.cc.cc";
  
  private String fakeSF1 = "zzz.zz.ZzzZzzZz";
  private String fakeSF2 = "yyy.yy.yy.yy.YYYYyyYy";
  
  private String memInitialCtxFactory = "test.thesis.jndi.provider.context.InitialInMemoryContextFactory";
  private String serviceURLCtxFactory = "test.thesis.jndi.provider.context.ServiceURLContextFactory";
  private String serviceURLctxPackage = "test.thesis.jndi.provider.context";
  
  private String bookOF = "test.thesis.jndi.provider.semantics.book.BookObjectFactory";
  private String bookSF = "test.thesis.jndi.provider.semantics.book.BookStateFactory";
  
  private String songOF = "test.thesis.jndi.provider.semantics.song.SongObjectFactory";
  private String songSF = "test.thesis.jndi.provider.semantics.song.SongStateFactory";
  
  private String bookName = "book";
  private String songURLName = "service:/song";
  
  private Properties bookInfo = new Properties();
  private Properties songInfo = new Properties();
  
  static {
	try {
	  ProviderManager.getInstance().registerProvider(new InMemoryContextProvider());
	} catch (Throwable e) {
	  e.printStackTrace();
	}
  }
  
  public static void registerProviders() {
  	try {
  	  ProviderManager.getInstance().registerProvider(new InMemoryContextProvider());
  	  ProviderManager.getInstance().registerProvider(new BookProvider());
  	} catch (Throwable e) {
  	  e.printStackTrace();
  	}
  }
  
  public void bindTest() {
	Hashtable env = new Hashtable();
	env.put(Context.INITIAL_CONTEXT_FACTORY, memInitialCtxFactory);
	env.put(Context.OBJECT_FACTORIES, fakeOF1 + ":" + fakeOF2 + ":" + bookOF + ":" + songOF);
	env.put(Context.STATE_FACTORIES, DispatchStateFactory.class.getName());
	env.put(DispatchStateFactory.EXTRA_STATE_FACTORIES, fakeSF1 + ":" + fakeSF2 + ":" + bookSF + ":" + songSF);
	env.put(Context.URL_PKG_PREFIXES, fakeURLpkg3 + ":" + serviceURLctxPackage);
	
	// first set the context classloader to be the oader we created for this bind test
	Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
	System.out.println("[TestClient] get InitialContext ... ");
	InitialContext ctx;
	try {
	  ctx = new InitialContext(env);
	  System.out.print(ctx);
	  assertNotNull(ctx);
	} catch (NamingException e) {
	  System.out.println("[TestClient] Cannot get InitialContext -> exit");
	  e.printStackTrace();
	  throw new AssertionError("Cannot get InitialContext because of [" + e + "]");
	}
	
	System.out.println("[TestClient] try to work with URL first");
	
	System.out.println("[TestClient] bind object with name " + songURLName);
	songInfo.put(Song.AUTHOR_PROPERTY_NAME, "Queen");
	songInfo.put(Song.GENRE_PROPERTY_NAME, "rock");
	songInfo.put(Song.TITLE_PROPERTY_NAME, "I'm going slightly mad");
	songInfo.put(Song.YEAR_PROPERTY_NAME, "1991");
	
	try {
	  ctx.rebind(songURLName, new Song(new SongInfo(songInfo)));
	  NamingEnumeration ne = ctx.list("service:/");
	  NameClassPair element = null;
	  assertTrue(ne.hasMoreElements());
	  while (ne.hasMoreElements()) {
		element = (NameClassPair) ne.nextElement();
		System.out.println("-> " + element);
	  }
	} catch (NamingException e) {
	  System.out.println("[TestClient] Cannot bind song");
	  e.printStackTrace();
	  throw new AssertionError("Cannot bind song object because of [" + e + "]");
	}
	
	System.out.println("[TestClient] lookup object with name " + songURLName);
	Object result = null;
	try {
	  result = ctx.lookup(songURLName);
	  System.out.println("[TestClient] object got: " + result);
	  assertNotNull(result);
	  assertTrue("Looked up object must be instance of Song", result instanceof Song);
	  assertTrue("Song properties got are the same as the ones we specified on bind", ((Song) result).getInfo().getProperties().equals(songInfo));
	  System.out.println("[TestClient] test passed song info is: " + ((Song) result).getInfo());
	  System.out.println();
	} catch (NamingException e) {
	  System.out.println("[TestClient] Cannot lookup song");
	  e.printStackTrace();
	  throw new AssertionError("Cannot lookup song object because of [" + e + "]");
	}
	
	bookInfo.put(Book.AUTHOR_PROPERTY_NAME, "Joahn Rowling");
	bookInfo.put(Book.GENRE_PROPERTY_NAME, "adventure");
	bookInfo.put(Book.TITLE_PROPERTY_NAME, "Harry Potter and the deadly hallows");
	bookInfo.put(Book.YEAR_PROPERTY_NAME, "2007");
	try {
	  ctx.rebind(bookName, new Book(new BookInfo(bookInfo)));
	  NamingEnumeration ne = ctx.list("");
	  NameClassPair element = null;
	  assertTrue(ne.hasMoreElements());
	  while (ne.hasMoreElements()) {
		element = (NameClassPair) ne.nextElement();
		System.out.println("-> " + element);
	  }
	} catch (NamingException e) {
	  System.out.println("[TestClient] Cannot bind book -> exit");
	  e.printStackTrace();
	  throw new AssertionError("Cannot bind book object because of [" + e + "]");
	}
	
	System.out.println("[TestClient] lookup object with name " + bookName);
	
	try {
	  result = ctx.lookup(bookName);
	  System.out.println("[TestClient] object got: " + result);
	  assertNotNull(result);
	  assertTrue("Looked up object must be instance of Book", result instanceof Book);
	  assertTrue(((Book) result).getInfo().getProperties().equals(bookInfo));
	  System.out.println("[TestClient] test passed book info is: " + ((Book) result).getInfo());
	  System.out.println();
	} catch (NamingException e) {
	  System.out.println("[TestClient] Cannot lookup book -> exit");
	  e.printStackTrace();
	  throw new AssertionError("Cannot lookup song object because of [" + e + "]");
	}	
  }
  
  public static void main(String[] args) {
	(new TestClient()).bindTest();
  }

}
