package test.thesis.jndi.provider.context;

import java.util.Hashtable;

import javax.naming.ConfigurationException;
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.NamingException;
import javax.naming.spi.ObjectFactory;

public class ServiceURLContextFactory implements ObjectFactory {

  /**
   *  The "service" url is of the form service:/<name>.
   */
  public Object getObjectInstance(Object urlInfo, Name name, Context nameCtx, Hashtable env) throws Exception {

	// Case 1: urlInfo is null
	// This means to create a URL context that can accept 
	// arbitrary "service" URLs.
	if (urlInfo == null) {
	  //System.out.println("[ServiceURLContextFactory] create a URL context that can accept arbitrary \"service\" URLs.");
	  return createURLContext(env);
	}

	// Case 2: urlInfo is a single string
	// This means to create/get the object named by urlInfo
	if (urlInfo instanceof String) {
	  Context urlCtx = createURLContext(env);
	  try {
		//System.out.println("[ServiceURLContextFactory] create/get the context named by " + urlInfo);
		return urlCtx.lookup((String)urlInfo);
	  } finally {
		urlCtx.close();
	  }
	} 

	// Case 3: urlInfo is an array of strings
	// This means each entry in array is equal alternative; create/get
	// the object named by one of the URls
	if (urlInfo instanceof String[]) {

	  // Try each URL until lookup() succeeds for one of them.
	  // If all URLs fail, throw one of the exceptions arbitrarily.
	  String[] urls = (String[])urlInfo;
	  if (urls.length == 0) {
		throw (new ConfigurationException("[ServiceURLContextFactory] empty URL array provided"));
	  }
	  System.out.println("[ServiceURLContextFactory] create/get the context named by on of the names in url array provided");
	  Context urlCtx = createURLContext(env);
	  try {
		NamingException ne = null;
		for (int i = 0; i < urls.length; i++) {
		  try {
			System.out.println("[ServiceURLContextFactory] try to get the context named by " + urlInfo);
			return urlCtx.lookup(urls[i]);
		  } catch (NamingException e) {
			System.out.println("[ServiceURLContextFactory] NamingException caugth " + e);
			ne = e;
		  }
		}
		throw ne;
	  } finally {
		urlCtx.close();
	  }
	} 

	// Case 4: urlInfo is of an unknown type
	// Provider-specific action: reject input

	throw new IllegalArgumentException("[ServiceURLContextFactory] argument must be a service URL string or an array of them");
  }

  protected Context createURLContext(Hashtable env) {
	return new ServiceSupportContext(env);
  }
  
}


