package test.thesis.jndi.provider.client;

import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Collection;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

public class JNDITest extends TestCase {
  
  private static ClassLoader parent = null;
  
  public JNDITest(String test) {
	super(test);
  } 
   
  public void testInitializeProviderManager() {
	 
	 System.out.println("Check if the provider is already registered");
	 assertFalse(javax.naming.spi.NamingManager.hasInitialContextFactoryBuilder());
	 
	 try {
	   System.out.println("Create separate classloader which can access ProviderManager module"); 
 	   URL[] urls = new URL[] {new URL("file:/D:/develop/workspace/JNDI_Provider_Module/bin/")};
       parent = new URLClassLoader(urls);
       assertNotNull(parent);
       
       Class c = parent.loadClass("thesis.jndi.provider.impl.ProviderResolver");
       Object o = c.newInstance();
       System.out.println("ProviderResolver instance - " + o);
       Method init =  c.getMethod("init", null);
       System.out.println("method init - "+ init);
       init.invoke(o, null);
       System.out.println("method is invoked");
      
       assertTrue(javax.naming.spi.NamingManager.hasInitialContextFactoryBuilder());
       System.out.println("Initialization was successful!");
   } catch (Exception e) {
     System.out.println("Error occured while initializing ProviderResolver. Exception is : " + e.toString());
     e.printStackTrace();
     throw new AssertionError("Cannot initialize ProviderManager.");
   }
   
  }
  
  public void testBind2() {
	try {
	  if (parent == null) {
		System.out.println("Create separate classloader which can access ProviderManager module"); 
		URL[] urls = new URL[] {new URL("file:/D:/develop/workspace/JNDI_Provider_Module/bin/")};
		parent = new URLClassLoader(urls);
		assertNotNull(parent);
	  }
	  
	  System.out.println("Create new classloader for the bind test which has access to ProviderManager but no access to the lookup client");
	  URL[] urls = new URL[] {new URL("file:D:/develop/workspace/JNDI_Provider_Test/")};
	  ClassLoader cl = null;
	  if (parent != null) {
        cl = new URLClassLoader(urls, parent);
	  } else {
		cl = new URLClassLoader(urls);
	  }
	  assertNotNull(cl);
      System.out.println(cl.getResource("test/thesis/jndi/provider/client/LookupClient.class"));
  	  
      Class c = cl.loadClass("test.thesis.jndi.provider.client.TestClient");
      System.out.println("Load class implementing bint test - " + c);
      Object o = c.newInstance();
      Method lookupTest =  c.getMethod("bindTest", null);
      lookupTest.invoke(o, null);
      System.out.println("method bindTest is invoked");
      
    } catch (Exception e) {
      e.printStackTrace();
      throw new AssertionError("Cannot invoke bind test because of [" + e  + "]");
    }
  }
  
  public void testLookup1() {
	(new LookupClient()).lookupTest();
  }
  
  public void testBind1() {
	(new LookupClient()).localBindTest();
  }
  
  public void testLookup2() {
	(new LookupClient()).lookupTest();
  }

  public static Test suite(){	
	TestSuite suite = new TestSuite(); 
	
	TestSuite plainJNDI = new TestSuite("Standard JNDI Approach");
	plainJNDI.addTest(new JNDITest("testBind1"));
	plainJNDI.addTest(new JNDITest("testLookup1"));
	
	TestSuite extendedJNDI = new TestSuite("JNDI SPI Extention");
	extendedJNDI.addTest(new JNDITest("testInitializeProviderManager"));	
	extendedJNDI.addTest(new JNDITest("testBind2"));
	extendedJNDI.addTest(new JNDITest("testLookup2"));
	
	suite.addTest(plainJNDI);
	suite.addTest(extendedJNDI);
	
	
	return suite;
  }
  
    
   
}
