package test.thesis.jndi.provider.client;

import java.util.Hashtable;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

import junit.framework.AssertionFailedError;
import junit.framework.TestCase;

public class LookupClient extends TestCase {


  private String fakeOF1 = "aaa.aaa.aaaaa.AAAAA";
  private String fakeOF2 = "b.bb.bbb.bbbb.BB";

  private String fakeURLpkg3 = "ccccccc.cc.cc.cc";

  private String fakeSF1 = "zzz.zz.ZzzZzzZz";
  private String fakeSF2 = "yyy.yy.yy.yy.YYYYyyYy";

  private String memInitialCtxFactory = "test.thesis.jndi.provider.context.InitialInMemoryContextFactory";
  private String serviceURLCtxFactory = "test.thesis.jndi.provider.context.ServiceURLContextFactory";
  private String serviceURLctxPackage = "test.thesis.jndi.provider.context";

  private String bookOF = "test.thesis.jndi.provider.semantics.book.BookObjectFactory";
  private String bookSF = "test.thesis.jndi.provider.semantics.book.BookStateFactory";

  private String songOF = "test.thesis.jndi.provider.semantics.song.SongObjectFactory";
  private String songSF = "test.thesis.jndi.provider.semantics.song.SongStateFactory";

  private String bookName = "book";
  private String songURLName = "service:/song";
  
  private Properties bookInfo = new Properties();
  private Properties songInfo = new Properties();

  public void lookupTest() {
	Hashtable env = new Hashtable();
	env.put(Context.INITIAL_CONTEXT_FACTORY, memInitialCtxFactory);
	env.put(Context.OBJECT_FACTORIES, fakeOF1 + ":" + fakeOF2 + ":" + bookOF + ":" + songOF);
	env.put(Context.STATE_FACTORIES, fakeSF1 + ":" + fakeSF2 + ":" + bookSF + ":" + songSF);
	env.put(Context.URL_PKG_PREFIXES, fakeURLpkg3 + ":" + serviceURLctxPackage);

	System.out.println("[LookupClient] get InitialContext ... ");
	InitialContext ctx;
	try {
	  ctx = new InitialContext(env);
	  System.out.print(ctx);
	} catch (NamingException e) {
	  System.out.println("[LookupClient] Cannot get InitialContext -> exit");
	  e.printStackTrace();
	  throw new AssertionFailedError("Cannot get InitialContext because of [" + e + "]");
	}

	System.out.println("[LookupClient] try to work with URL first");

	System.out.println("[LookupClient] lookup object with name " + songURLName);
	try {
	  Object o = ctx.lookup(songURLName);
	  System.out.println("[LookupClient] object got: " + o);
	  assertNotNull(o);
	  System.out.println("[LookupClient] list objects in : service:/");
	  NamingEnumeration ne = ctx.list("service:/");
	  NameClassPair element = null;
	  while (ne.hasMoreElements()) {
		element = (NameClassPair) ne.nextElement();
		System.out.println("-> " + element);
	  }
	} catch (NamingException e) {
	  System.out.println("[LookupClient] Cannot lookup song");
	  e.printStackTrace();
	  throw new AssertionFailedError("Cannot lookup object [song] because of [" + e + "]");
	}

//	System.out.println("[LookupClient] lookup object with name " + songURLName);
//	Object result = null;
//	try {
//	result = ctx.lookup(songURLName);
//	System.out.println("[LookupClient] object got: " + result);
//	if(result instanceof Song) {
//	if(((Song) result).getInfo().getProperties().equals(songInfo)) {
//	System.out.println("[LookupClient] test passed song info is: " + ((Song) result).getInfo());
//	System.out.println();
//	}
//	}

//	} catch (NamingException e) {
//	System.out.println("[LookupClient] Cannot lookup song");
//	e.printStackTrace();
//	}

//	bookInfo.put(Book.AUTHOR_PROPERTY_NAME, "Joahn Rowling");
//	bookInfo.put(Book.GENRE_PROPERTY_NAME, "adventure");
//	bookInfo.put(Book.TITLE_PROPERTY_NAME, "Harry Potter and the deadly hallows");
//	bookInfo.put(Book.YEAR_PROPERTY_NAME, "2007");
	try {
	  System.out.println("[LookupClient] lookup object with name " + bookName);
	  Object o = ctx.lookup(bookName);
	  System.out.println("[LookupClient] object got: " + o);
	  assertNotNull(o);
	  System.out.println("[LookupClient] list objects in : []");
	  NamingEnumeration ne = ctx.list("");
	  NameClassPair element = null;
	  while (ne.hasMoreElements()) {
		element = (NameClassPair) ne.nextElement();
		System.out.println("-> " + element);
	  }

	} catch (NamingException e) {
	  System.out.println("[LookupClient] Cannot lookup book -> exit");
	  e.printStackTrace();
	  throw new AssertionFailedError("Cannot lookup object [book] because of [" + e + "]");
	}

//	System.out.println("[LookupClient] lookup object with name " + bookName);

//	try {
//	result = ctx.lookup(bookName);
//	System.out.println("[LookupClient] object got: " + result);
//	if(result instanceof Book) {
//	if(((Book) result).getInfo().getProperties().equals(bookInfo)) {
//	System.out.println("[LookupClient] test passed book info is: " + ((Book) result).getInfo());
//	System.out.println();
//	}
//	}

//	} catch (NamingException e) {
//	System.out.println("[LookupClient] Cannot lookup book -> exit");
//	e.printStackTrace();
//	}	
  }

  public void localBindTest() {
	Hashtable env = new Hashtable();
	env.put(Context.INITIAL_CONTEXT_FACTORY, memInitialCtxFactory);
	env.put(Context.OBJECT_FACTORIES, fakeOF1 + ":" + fakeOF2 + ":" + bookOF + ":" + songOF);
	env.put(Context.URL_PKG_PREFIXES, fakeURLpkg3 + ":" + serviceURLctxPackage);

	System.out.println("[TestClient] get InitialContext ... ");
	InitialContext ctx;
	try {
	  ctx = new InitialContext(env);
	  System.out.print(ctx);
	  assertNotNull(ctx);
	} catch (NamingException e) {
	  System.out.println("[BindClient] Cannot get InitialContext -> exit");
	  e.printStackTrace();
	  throw new AssertionFailedError("Cannot get InitialContext because of [" + e + "]");
	}

	System.out.println("[BindClient] try to work with URL first");

	System.out.println("[BindClient] bind object with name " + songURLName);
	
	try {
	  ctx.rebind(songURLName, new String("Some Object"));
	  NamingEnumeration ne = ctx.list("service:/");
	  NameClassPair element = null;
	  assertTrue(ne.hasMoreElements());
	  while (ne.hasMoreElements()) {
		element = (NameClassPair) ne.nextElement();
		System.out.println("-> " + element);
	  }
	} catch (NamingException e) {
	  System.out.println("[BindClient] Cannot bind song");
	  e.printStackTrace();
	  throw new AssertionFailedError("Cannot bind song object because of [" + e + "]");
	}

	System.out.println("[BindClient] lookup object with name " + songURLName);
	Object result = null;
	try {
	  result = ctx.lookup(songURLName);
	  System.out.println("[BindClient] object got: " + result);
	  assertNotNull(result);
	  assertTrue("Looked up object must be instance of String", result instanceof String);
	  System.out.println("[BindClient] test passed song info is: " + result);
	  System.out.println();
	} catch (NamingException e) {
	  System.out.println("[BindClient] Cannot lookup song");
	  e.printStackTrace();
	  throw new AssertionError("Cannot lookup song object because of [" + e + "]");
	}

	try {
	  ctx.rebind(bookName, new String("Some book"));
	  NamingEnumeration ne = ctx.list("");
	  NameClassPair element = null;
	  assertTrue(ne.hasMoreElements());
	  while (ne.hasMoreElements()) {
		element = (NameClassPair) ne.nextElement();
		System.out.println("-> " + element);
	  }
	} catch (NamingException e) {
	  System.out.println("[BindClient] Cannot bind book -> exit");
	  e.printStackTrace();
	  throw new AssertionFailedError("Cannot bind book object because of [" + e + "]");
	}

	System.out.println("[BindClient] lookup object with name " + bookName);

	try {
	  result = ctx.lookup(bookName);
	  System.out.println("[BindClient] object got: " + result);
	  assertNotNull(result);
	  assertTrue("Looked up object must be instance of String", result instanceof String);
	  System.out.println("[BindClient] test passed book info is: " + result);
	  System.out.println();
	} catch (NamingException e) {
	  System.out.println("[BindClient] Cannot lookup book -> exit");
	  e.printStackTrace();
	  throw new AssertionFailedError("Cannot lookup song object because of [" + e + "]");
	}	
  }

  public static void main(String[] args) {
	(new LookupClient()).lookupTest();
  }




}
